@@ -5,7 +5,7 @@
  * object level logging, and fully-qualified object names for all DML and DDL
  * statements where possible (See README.md for details).
  *
- * Copyright (c) 2014-2021, PostgreSQL Global Development Group
+ * Copyright (c) 2014-2022, PostgreSQL Global Development Group
  *------------------------------------------------------------------------------
  */
 #include "postgres.h"
@@ -19,6 +19,7 @@
 #include "catalog/pg_class.h"
 #include "catalog/namespace.h"
 #include "commands/dbcommands.h"
+#include "commands/extension.h"
 #include "catalog/pg_proc.h"
 #include "commands/event_trigger.h"
 #include "executor/executor.h"
@@ -493,6 +494,19 @@ log_audit_event(AuditEventStackItem *stackItem)
     MemoryContext contextOld;
     StringInfoData auditStr;
 
+    /*
+     * Skip logging script statements if an extension is currently being created
+     * or altered. PostgreSQL reports the statement text for each statement in
+     * the script as the entire script text, which can blow up the logs. The
+     * create/alter statement will still be logged.
+     *
+     * Since a superuser is responsible for determining which extensions are
+     * available, and in most cases installing them, it should not be necessary
+     * to log each statement in the script.
+     */
+    if (creating_extension)
+        return;
+
     /* If this event has already been logged don't log it again */
     if (stackItem->auditEvent.logged)
         return;
@@ -1048,7 +1062,7 @@ log_select_dml(Oid auditOid, List *rangeTabls)
         /*
          * We don't have access to the parsetree here, so we have to generate
          * the node type, object type, and command tag by decoding
-         * rte->requiredPerms and rte->relkind. For updates we also check 
+         * rte->requiredPerms and rte->relkind. For updates we also check
          * rellockmode so that only true UPDATE commands (not
          * SELECT FOR UPDATE, etc.) are logged as UPDATE.
          */
@@ -1536,6 +1550,18 @@ pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
             !IsAbortedTransactionBlockState())
             log_audit_event(stackItem);
 
+        /*
+         * If this is a create/alter extension command log it before calling
+         * the next ProcessUtility hook. Otherwise, any warnings will be emitted
+         * before the create/alter is logged and errors will prevent it from
+         * being logged at all.
+         */
+        if (auditLogBitmap & LOG_DDL &&
+            (stackItem->auditEvent.commandTag == T_CreateExtensionStmt ||
+                stackItem->auditEvent.commandTag == T_AlterExtensionStmt) &&
+            !IsAbortedTransactionBlockState())
+            log_audit_event(stackItem);
+
         /*
          * A close will free the open cursor which will also free the close
          * audit entry. Immediately log the close and set stackItem to NULL so
@@ -1655,7 +1681,9 @@ pgaudit_ddl_command_end(PG_FUNCTION_ARGS)
         CreateCommandTag(eventData->parsetree);
 
     /* Return objects affected by the (non drop) DDL statement */
-    query = "SELECT UPPER(object_type), object_identity, UPPER(command_tag)\n"
+    query = "SELECT pg_catalog.upper(object_type),\n"
+            "       object_identity,\n"
+            "       pg_catalog.upper(command_tag)\n"
             "  FROM pg_catalog.pg_event_trigger_ddl_commands()";
 
     /* Attempt to connect */
@@ -1754,11 +1782,11 @@ pgaudit_sql_drop(PG_FUNCTION_ARGS)
     contextOld = MemoryContextSwitchTo(contextQuery);
 
     /* Return objects affected by the drop statement */
-    query = "SELECT UPPER(object_type),\n"
-        "       object_identity\n"
-        "  FROM pg_catalog.pg_event_trigger_dropped_objects()\n"
-        " WHERE lower(object_type) <> 'type'\n"
-        "   AND schema_name <> 'pg_toast'";
+    query = "SELECT pg_catalog.upper(object_type),\n"
+            "       object_identity\n"
+            "  FROM pg_catalog.pg_event_trigger_dropped_objects()\n"
+            " WHERE pg_catalog.lower(object_type) operator(pg_catalog.<>) 'type'\n"
+            "   AND schema_name operator(pg_catalog.<>) 'pg_toast'";
 
     /* Attempt to connect */
     result = SPI_connect();
